/*
 * Serial Debugging Class
 *
 * S.Holzheu (holzheu@bayceer.uni-bayreuth.de)
 *
 * This is a Class sending BayEOS-Frames in a human readable form
 * helps debugging Arduino-BayEOS-Projects...
 *
 */

#ifndef BayDEBUG_h
#define BayDEBUG_h

#include <BayEOS.h>

class BayDebug : protected HardwareSerial, public BayEOS {
public:
	/**
	 * Constructor
	 */
	BayDebug(HardwareSerial &serial=Serial);

	/**
	 * Send the BayEOS frame to Serial
	 * will return error according to modus
	 */
	uint8_t sendPayload(void);

	/**
	 * Begin Serial connection...
	 * modus=0 -> Print out Hex
	 * modus=1 -> Print out frames in human readable form
	 * modus=2 -> Print out Hex with send error every second frame
	 * modus=3 -> Print out frames in human readable form
	 *            with send error every second frame
	 *
	 **/
	void begin(long baud,uint8_t modus=3);
	uint8_t _modus;
private:
	void parse(uint8_t offset=0);
	void parseDataFrame(uint8_t offset);
	uint8_t _error_next;

};


#endif
