/*----------------------------------------------------
Simple BayXBee router designed for Arduino-Mega
using two serial ports and a RAM for buffering
when destination XBee is not available.
----------------------------------------------------*/

#define RX_SERIAL Serial
#define TX_SERIAL Serial1 
/*
To save ram you can define RX_PANID here.
The programm will set rx_panid without querying the bee

#define RX_PANID 10002
*/


#include <XBee.h>
#include <BayEOS.h>
#include <BayXBee.h>
#include <BayEOSBuffer.h>
#include <BayEOSBufferRAM.h>


XBee xbee_rx = XBee();
BayXBee xbee_tx = BayXBee();

BayEOSBufferRAM  myBuffer;

// create reusable response objects for responses we expect to handle 
Rx16Response rx16 = Rx16Response();


uint8_t data = 0;
uint8_t i;
uint16_t rx_panid;

void setup() {
  xbee_tx.setSerial(TX_SERIAL); 
  xbee_rx.setSerial(RX_SERIAL);
  xbee_tx.begin(38400);
  xbee_rx.begin(38400);
  
  myBuffer=BayEOSBufferRAM(3000);
  xbee_tx.setBuffer(myBuffer);
  
#if RX_PANID
  rx_panid=RX_PANID;
#else
  while(! rx_panid) rx_panid=getPANID(xbee_rx);
#endif
}



void loop() {
   
    xbee_rx.readPacket();
   
    if (xbee_rx.getResponse().isAvailable()) {
      // got something
     
      if (xbee_rx.getResponse().getApiId() == RX_16_RESPONSE ) {
        // got a rx16 packet
        xbee_rx.getResponse().getRx16Response(rx16);
        xbee_tx.startRoutedFrame(rx16.getRemoteAddress16(),rx_panid);

		for(i=0; i<rx16.getDataLength();i++){
			xbee_tx.addToPayload(rx16.getData(i));
		}
		xbee_tx.sendOrBuffer();
		
      }
    }
	//Code zum Nachsenden von gepufferten Daten
	//Immer nur das nächste Paket...
    xbee_tx.sendFromBuffer(); 
 }

