/*
 * Arduino BayEth
 *
 * S.Holzheu (holzheu@bayceer.uni-bayreuth.de)
 *
 * Class for sending HTTP-Post-Request to a BayEOS Gateway
 *
 */

#ifndef BayEth_h
#define BayEth_h

#ifndef BayETH_BUFFER
#define BayETH_BUFFER 137
#endif

#include <Ethernet.h>
#include <SPI.h>
#include "../BayEOS/BayEOS.h"
#include "../Base64/Base64.h"

class BayEth : protected EthernetClient,public BayEOS {
public:
	/**
	 * Constructor
	 *
	 */
	BayEth(const uint8_t *server,const char *pw,const char *sender=0, const char *path="/", const uint16_t port=8090);

	/**
	 * Send the BayXBee payload as base64 encoded string
	 * returns 0 for success
	 * 1 == no success
	 * 2 == timeout
	 * 3 == no connect
	 */
	uint8_t sendPayload(void);

	/**
	 * Send several frames in one post request
	 * returns 0 for success
	 * 1 == no success
	 * 2 == timeout
	 * 3 == no connect
	 */
	uint8_t sendMultiFromBuffer(int maxsize=5000);

	int available(void){
		return EthernetClient::available();
	}

	uint8_t _serial_debug;
private:

	byte _server[4];
	char _server_path[17];
	uint16_t _server_port;
	char _sender[17];
	char _password[9];
	char _base64buffer[BayETH_BUFFER];
	uint8_t _data_length;
	unsigned long _timeout;


};


#endif

