/*
Test programm for sending BayEOS Frames over Ethernet

Normal frames (27 Byte) will look like
            01 41 00 0000803f 01 0000803f 02 0000803f 03 0000803f 04 0000803f

Delayed frames (32 Byte) will look like
07 xxxxxxxx 01 41 00 0000803f 01 0000803f 02 0000803f 03 0000803f 04 0000803f

Password is "bayeos"

Buffer size should be something like 
days * 24 * 3600 * (framesize+5)/sampling intervall

more than 48 days will not work (millis() wrap around)


*/

#include <Ethernet.h>
#include <SPI.h>
#include <Base64.h>
#include <BayEOS.h>
#include <BayEth.h>
#include <SdFat.h>
#include <BayEOSBuffer.h>
#include <BayEOSBufferSDFat.h>

#define SerialDebug 0
BayEOSBufferSDFat myBuffer;

#define SerialPrintLN(x) SerialPrintLN_P(PSTR(x))
void SerialPrintLN_P(PGM_P str ) {
  for (uint8_t c; (c = pgm_read_byte(str)); str++) Serial.write(c);
  Serial.println();
}

const byte* new_addr;

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
byte ip[] = { 192, 168, 1, 2 };
byte server[]={ 192, 168, 1, 1 };

BayEth client=BayEth( server,"bayeos");

unsigned long last_data=0;


	


void setup(void){
  pinMode(53, OUTPUT);
  pinMode(4, OUTPUT);
  
  Serial.begin(9600);
  
#if SerialDebug
  SerialPrintLN("Initializing Ethernet...");
#endif
  Ethernet.begin(mac, ip);
  client._serial_debug=SerialDebug;

#if SerialDebug
  SerialPrintLN("Done");
  SerialPrintLN("Initializing SD card...");
#endif

  if (!SD.begin(4)) {
#if SerialDebug
  	SerialPrintLN("No SD");
#endif
  	
  	delay(1000);
    return;
  }
#if SerialDebug
  SerialPrintLN("Done");
#endif

  myBuffer = BayEOSBufferSDFat(100000000); //create a buffer file (max. 100MB) on SD 
  client.setBuffer(myBuffer); 
}

void loop(void){
  if((millis()-last_data)>5000){
    last_data=millis();
    client.startDataFrame(0x41);
    for(uint8_t i=0;i<5;i++){
      client.addToPayload(i);
      client.addToPayload((float) i);
    }
   
    client.sendOrBuffer();

	client.startFrame((uint8_t) BayEOS_Message);
	client.addToPayload(":-)");
	
    client.sendOrBuffer();

	client.startFrame((uint8_t) BayEOS_ErrorMessage);
	client.addToPayload(":-(");
	
    client.sendOrBuffer();

  }
  client.sendFromBuffer();
}

