/**
Router - XBee-Ethernet with SD-Card Buffer
Please note: 
Depending on the size of the RX-Serial-Buffer there is a 
it is possible that RX-Packets get lost, because RX-Serial-Buffer
fills up while Arudino is sending.
Normal buffer size is 64 byte.
You can increase setting the SERIAL_BUFFER_SIZE in
hardware/arduino/cores/arduino/HardwareSerial.cpp
*/
#include <XBee.h>
#include <BayEOS.h>
#include <BayXBee.h>
#include <Ethernet.h>
#include <SPI.h>
#include <Base64.h>
#include <BayEth.h>
#include <SD.h>
#include <BayEOSBuffer.h>
#include <BayEOSBufferSD.h>

/* 
Please define your MAC, IP and Server, password, Name
*/
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0D, 0x10, 0xB4 };
byte ip[] = { 192, 168, 1, 2 };
byte server[]={ 192, 168, 1, 1 };
BayEth client=BayEth( server,"bayeos","Ethernet-Router");

#define RX_SERIAL Serial


XBee xbee_rx = XBee();
BayEOSBufferSD myBuffer;
Rx16Response rx16 = Rx16Response();
uint16_t rx_panid;

unsigned long last_alive;

void setup(void){
  pinMode(53, OUTPUT);
  pinMode(4, OUTPUT);
  
  
  Ethernet.begin(mac, ip);
  xbee_rx.setSerial(RX_SERIAL);
  xbee_rx.begin(38400);	


//Use 4 for Ethernet-Shield
// "A11" for Mega-XBee-Shield
  if (!SD.begin(A11)) {
  	client.sendError("No SD.");
  }
  myBuffer = BayEOSBufferSD(10000000);
  
  
  while(! rx_panid) rx_panid=getPANID(xbee_rx);
  client.setBuffer(myBuffer);
}

void loop(void){
    if(millis()-last_alive>30000){
      last_alive=millis();
      client.startDataFrame(BayEOS_Float32le);
      client.addToPayload((uint8_t) 0);
      client.addToPayload((float) millis()/1000);
      client.sendPayload();
    }
   xbee_rx.readPacket();
   
    if (xbee_rx.getResponse().isAvailable()) {
      // got something
      if (xbee_rx.getResponse().getApiId() == RX_16_RESPONSE ) {
        // got a rx16 packet
        xbee_rx.getResponse().getRx16Response(rx16);
        if (xbee_rx.getResponse().isError()) {
		// get the error code
			client.startFrame(BayEOS_ErrorMessage);
			client.addToPayload("RX-ERROR:"+xbee_rx.getResponse().getErrorCode());
	    } else {
//          client.startRoutedFrame(rx16.getRemoteAddress16(),rx_panid,rx16.getRssi());
          client.startRoutedFrame(rx16.getRemoteAddress16(),rx_panid);
      	  for(uint8_t i=0; i<rx16.getDataLength();i++){
			client.addToPayload(rx16.getData(i));
		  }
		}
		client.sendOrBuffer();
      }
    }
    client.sendFromBuffer();
 
}

