#include "BayGPRS.h"
#define printP(x) printPGM(PSTR(x))
#define printlnP(x) printlnPGM(PSTR(x))
#define wait_for(x,y) wait_forPGM(PSTR(x),y)
#define SerialprintP(x) serialprintPGM(PSTR(x))
#define SerialprintlnP(x) serialprintlnPGM(PSTR(x))
/*
 * RTC-Clock with SIM900 + millis() implementation
 * Please note: Direkt getting time for SIM900 will not work,
 * as SIM900 will not accept commands while composing HTML-POST-Request!
 *
 */

void RTC_SIM900::adjust(const DateTime& dt) {
	last_millis=millis();
    offset = dt.get();
}

DateTime RTC_SIM900::now() {
	unsigned long diff=(millis()-last_millis)/1000L;
	if(diff>1000000L){
		offset+=diff;
		last_millis=millis();
		diff=0;
	}
    return offset+diff;
}


BayGPRS::BayGPRS(HardwareSerial &serial,uint8_t powerPin):HardwareSerial(serial){
	_powerPin=powerPin;
}

BayGPRS::BayGPRS(const char *url,const char *pw,const char *sender,const char *apn, HardwareSerial &serial,uint8_t powerPin):HardwareSerial(serial){
	memset(_config_buffer,'\0',BayGPRS_CONFIG_SIZE);
	uint8_t offset=0;
	offset+=addToConfigBuffer(offset,url);
	offset+=addToConfigBuffer(offset,pw);
	offset+=addToConfigBuffer(offset,sender);
	offset+=addToConfigBuffer(offset,apn);
	_powerPin=powerPin;
	setConfigPointers();
}
uint8_t BayGPRS::begin(long baud){
	_baud=baud;
	return init();
}

void BayGPRSInterface::serialprintPGM(const char *str){
	char c;
	while (true) {
		c=pgm_read_byte(str);
		if (!c) break;
	    Serial.write(c);
	    str++;
	}
}
void BayGPRSInterface::serialprintlnPGM(const char *str){
	serialprintPGM(str);
	Serial.println();
}


void BayGPRSInterface::printPGM(const char *str){
	char c;
	while (true) {
		c=pgm_read_byte(str);
		if (!c) break;
#if BayGPRS_DEBUG_INPUT
		Serial.write(c);
#endif

		write(c);
	    str++;
	}
}
void BayGPRSInterface::printlnPGM(const char *str){
	printPGM(str);
	println();
}

void BayGPRSInterface::skipChars(void){
    do{
  	  while(i_available()){
 #if BayGPRS_DEBUG_OUTPUT
	Serial.print((char) read());
#else
	  read();
#endif

  	  }
  	  delay(1);
	} while(i_available());

}


uint8_t BayGPRSInterface::sendATE0(void){
	skipChars();
	printlnP("ATE0");
	return(wait_forOK(1000));
}

uint8_t BayGPRSInterface::sendSMS(const String &phone, const String &sms){
	sendATE0();
	printlnP("AT+CMGF=1");//humanreadable SMS
	if(wait_forOK(2000)) return 2;
	printP("AT+CMGS=\"");//Target phone number
	print(phone);
	printlnP("\"");
	if(wait_for("> ",2000)) return 2;
	print(sms);
	write(0x1a);
	if(wait_for("+CMGS: ",2000)) return 2;
	return 0;
}

uint8_t BayGPRSInterface::sendMultiFromBuffer(int maxsize){
	if(! _buffer->available()) return 0;
	uint8_t res;
	res=connect();
	if(res) return (res+2);

	unsigned long readpos=_buffer->readPos();

	int size=7+strlen(_sender)+1+9+strlen(_password);


	while(size<maxsize && readFromBuffer()){
		size+=1+15+base64_enc_len(getPacketLength());
		_buffer->next();
	}
	_buffer->seekReadPointer(readpos);
	printP("AT+HTTPDATA=");
	print(size);
	printlnP(",10000");
	if(wait_for("DOWNLOAD",20000)) return 2;

	printP("sender=");
 	print(_sender);
	printP("&password=");
 	print(_password);
	size=7+strlen(_sender)+1+9+strlen(_password);
	while(size<maxsize && readFromBuffer()){
		size+=1+15+base64_enc_len(getPacketLength());
	    printP("&bayeosframes[]=");
		base64_encode(_base64buffer,(char*) _payload,getPacketLength());
		_base64buffer[base64_enc_len(getPacketLength())]=0;
		print(_base64buffer);
		_buffer->next();
	}
	println();
	if(wait_forOK(30000)) return 2;
	printlnP("AT+HTTPACTION=1");
	res=wait_for("+HTTPACTION:1,200,",30000);

	if(res){
		_buffer->seekReadPointer(readpos);
		_tx_error_count++;
	}
	else _tx_error_count=0;
	return res;
}

uint8_t BayGPRSInterface::sendPayload(void){
#if BayGPRS_DEBUG_OUTPUT
	Serial.println("sendPayload");
#endif
	uint8_t res;
	res=connect();
	if(res) return (res+2);
	_data_length=base64_enc_len(getPacketLength());
	base64_encode(_base64buffer,(char*) _payload,getPacketLength());
	_base64buffer[_data_length]=0;
	_data_length+=7+strlen(_sender)+1+9+strlen(_password)+1+12;

	printP("AT+HTTPDATA=");
	print(_data_length);
	printlnP(",2000");
	if(wait_for("DOWNLOAD",2000)) return 2;

	printP("sender=");
 	print(_sender);
	printP("&password=");
 	print(_password);
    printP("&bayeosframe=");
    print(_base64buffer); //BASE64
    println();
    //wait_forOK(40000);
	if(wait_forOK(30000)) return 2;


	printlnP("AT+HTTPACTION=1");
//	if(wait_forOK(4000)) return 2;
	res=wait_for("+HTTPACTION:1,200,",30000);
	//return 0;
// 	println("AT+HTTPTERM");
//	wait_forOK(2000);
	if(res) _tx_error_count++;
	else _tx_error_count=0;
	return res;
}

uint8_t BayGPRSInterface::setClock(const char* time){
	sendATE0();
	printP("AT+CCLK=\"");
	print(time);
	printlnP("\"");
	SerialprintP("setting Clock to:");
	Serial.print(time);
	if(wait_forOK(1000)) SerialprintlnP("failed");
	else SerialprintlnP("ok");
	return 1;
	return wait_forOK(1000);
}

void BayGPRSInterface::serialPrintlnTime(void){
	sendATE0();
		printlnP("AT+CCLK?");
		wait_forPGM(PSTR("+CCLK: \""),3000,20,_base64buffer); //YY/MM/DD,HH:MM:SS+02
		//_base64buffer[20]=0;
		SerialprintP("Time:");
		Serial.println(_base64buffer);

}

void BayGPRSInterface::serialSetClock(void){
	do{
		delay(10);
		Serial.read();
	} while(Serial.available());
	while(true){
		serialPrintlnTime();
		SerialprintlnP("Enter new date+time or press e to exit");
		if(Serial.available()){
			readSerialInput();
			if(_SerialInputBuffer[0]=='e') return;
			else setClock(_SerialInputBuffer);
		}
		do{
			delay(10);
			Serial.read();
		} while(Serial.available());
		delay(2000);
	}

}

DateTime BayGPRSInterface::now(void){
	sendATE0();
	printlnP("AT+CCLK?");
	wait_forPGM(PSTR("+CCLK: \""),3000,20,_base64buffer); //YY/MM/DD,HH:MM:SS+02
//	Serial.print("LocalTime:");
//	Serial.println(_base64buffer);
	uint16_t y = atoi(_base64buffer+0) + 2000;
	uint8_t m = atoi(_base64buffer+3);
	uint8_t d = atoi(_base64buffer+6);
	uint8_t hh = atoi(_base64buffer+9);
	uint8_t mm = atoi(_base64buffer+12);
	uint8_t ss = atoi(_base64buffer+15);
	DateTime dt=DateTime (y, m, d, hh, mm, ss);
	dt=DateTime(dt.get()-(3600L*atoi(_base64buffer+17))); //Adjust for Timezone!
	return dt;
}


uint8_t BayGPRSInterface::getRSSI(void){
	sendATE0();
	printlnP("AT+CSQ");
	wait_forPGM(PSTR("+CSQ: "),500,2,_base64buffer);
	return (uint8_t)114 -(2*(uint8_t)atoi(_base64buffer));
}
void BayGPRSInterface::disconnect(void){
	sendATE0();
	printlnP("AT+SAPBR=0,1");
	if(wait_forOK(2000)==2) init();
	printlnP("AT+HTTPTERM");
	wait_forOK(1000);

}
uint8_t BayGPRSInterface::connect(void){
	uint8_t count=0;
	setup_start:
#if BayGPRS_DEBUG_OUTPUT
	Serial.print("-c");
#endif
	sendATE0();

	printlnP("AT+SAPBR=2,1");
	if(! wait_for("+SAPBR: 1,1",4000)){
		skipChars();
		return 0;
	}
	//Switch off modem after 30 consecutive
	if(_tx_error_count>30)
		softSwitch();

	// Init Modem
	if(init()) return 2;

	count++;
	printlnP("AT+SAPBR=0,1");
	wait_forOK(2000);

	printlnP("AT+HTTPTERM");
	wait_forOK(1000);
	if(count>100) return 1;


	printlnP("AT+SAPBR=3,1,\"CONTYPE\",\"GPRS\"");
	if(wait_forOK(400)) goto setup_start;
	printP("AT+SAPBR=3,1,\"APN\",\"");
	print(_apn);
	printlnP("\"");
	if(wait_forOK(400)) goto setup_start;
	printP("AT+SAPBR=3,1,\"USER\",\"");
	if(_prov_user[0]) print(_prov_user);
	printlnP("\"");
	if(wait_forOK(400)) goto setup_start;

	printP("AT+SAPBR=3,1,\"PWD\",\"");
	if(_prov_pw[0]) print(_prov_pw);
	printlnP("\"");
	if(wait_forOK(400)) goto setup_start;


	printlnP("AT+SAPBR=1,1");
	if(wait_forOK(60000)) goto setup_start;

	//HTTP-Connection
	printlnP("AT+HTTPINIT"); //Open a connection
	if(wait_forOK(2000)) goto setup_start;
	printlnP("AT+HTTPPARA=\"CID\",1");
	if(wait_forOK(2000)) goto setup_start;
	printlnP("AT+HTTPPARA=\"CONTENT\",\"application/x-www-form-urlencoded\"");
	if(wait_forOK(2000)) goto setup_start;

	printlnP("AT+HTTPPARA=\"UA\",\"BayEOS arduino\"");
	if(wait_forOK(2000)) goto setup_start;
	printP("AT+HTTPPARA=\"URL\",\"");
	print(_url);
	printlnP("\"");
	if(wait_forOK(2000)) goto setup_start;

	return 0;
}
uint8_t BayGPRSInterface::setBaudRate(long baud){
	long prev_baud=38400;
	set_baud:
	i_begin(prev_baud);
	printlnP("AT");
	delay(100);
	printlnP("AT+CSCLK=0"); //Auto-Sleepmode off!!
	if(wait_forOK(200)){
		prev_baud/=2;
		if(prev_baud>=4800)
			goto set_baud;
		else return 1;
	}
	printP("AT+IPR=");
	println(baud);
	wait_forOK(200);
	i_begin(baud);
	return 0;
}

uint8_t BayGPRSInterface::init(){
	i_begin(_baud);
	uint8_t count=0;
	init_start:
	printlnP("AT");
	delay(100);
	printP("AT+IPR=");
	println(_baud);
	wait_forOK(200);
	for(uint8_t i=0;i<5;i++){
		printlnP("ATE0"); //Command echo off
		if(! wait_forOK(200)){
			//communication ok!
			if(_pin[0]){
				printP("AT+CPIN=\"");
				print(_pin);
				println("\"");
#if BayGPRS_DEBUG_INPUT
				Serial.print(_pin);
				Serial.println("\"");
#endif
				wait_forOK(200);
				//if(wait_forOK(200)) return 2;
			}
			printlnP("AT+CLTS=1"); //Enable Local Network Time
			wait_forOK(200);
			//Attention: Setting AT+IPR=BAUD-Rate does not work with Auto-Sleepmode enabled!!
			printlnP("AT+CSCLK=2"); //Auto-Sleepmode
			if(! wait_forOK(200)) return 0;
		}
	}
	softSwitch();
	count++;
#if BayGPRS_DEBUG_OUTPUT
	Serial.print("i-");
#endif

	if(count>1) return 1;
	goto init_start;
}

void BayGPRSInterface::softSwitch(void){
	pinMode(_powerPin, OUTPUT);
	digitalWrite(_powerPin,LOW);
	delay(1000);
	digitalWrite(_powerPin,HIGH);
	delay(2500);
	digitalWrite(_powerPin,LOW);
	delay(3500);
	_tx_error_count=0;
}

uint8_t BayGPRSInterface::addToConfigBuffer(uint8_t offset,const char* str){
	if(strlen(str)>BayGPRS_CONFIG_SIZE -offset-1){
		strncpy(_config_buffer+offset,str,BayGPRS_CONFIG_SIZE -offset-1);
		_config_buffer[BayGPRS_CONFIG_SIZE -1]=0;
		return BayGPRS_CONFIG_SIZE -offset;
	} else {
		strcpy(_config_buffer+offset,str);
		return strlen(str)+1;
	}
}


void BayGPRSInterface::readSerialInput(uint8_t timeout){
	uint8_t offset=0;
	while(true){
		while(! Serial.available() && timeout>0){
			delay(1);
			timeout--;
		}
		if(Serial.available()){
			_SerialInputBuffer[offset]=Serial.read();
			if(_SerialInputBuffer[offset]=='\r' || _SerialInputBuffer[offset]=='\n'){
//				Serial.println("Got nl/cr");
				do{
					delay(10);
					Serial.read();
				} while(Serial.available());
				timeout=0;
			}

		}
		if(timeout==0){
			_SerialInputBuffer[offset]='\0';
//			Serial.println(_SerialInputBuffer);
			return;
		}
		if(offset<_SerialInputBufferSize) offset++;
	}
}

void BayGPRSInterface::setConfigFromSerial(uint8_t index,uint8_t timeout){
  readSerialInput(timeout);
  setConfig(_SerialInputBuffer,index);

}
void BayGPRSInterface::readConfigFromFile(const char *file){
	SdFile _f;
	_f.open(file, O_READ);
	uint8_t i=0;
	int c;
	while((i<BayGPRS_CONFIG_SIZE) && ((c=_f.read())!=-1)){
//		Serial.print((char) c);
		if(c=='|') c=0;
		_config_buffer[i]=(char) c;
		i++;
	}
	_f.close();
	setConfigPointers();
}


void BayGPRSInterface::readConfigFromEEPROM(int eeoffset){
	if(EEPROM.read(eeoffset)==0xff){
		//No valid config!!
		return;
	}
	for(uint8_t i=0;i<BayGPRS_CONFIG_SIZE;i++){
		_config_buffer[i]=EEPROM.read(eeoffset+i);
	}
	setConfigPointers();
}

void BayGPRSInterface::writeConfigToEEPROM(int eeoffset){
	for(uint8_t i=0;i<BayGPRS_CONFIG_SIZE;i++){
		EEPROM.write(eeoffset+i,_config_buffer[i]);
	}
}
void BayGPRSInterface::printConfigName(uint8_t index){
	switch(index){
	case 0:
		SerialprintP("URL");
		break;
	case 1:
		SerialprintP("PW");
		break;
	case 2:
		SerialprintP("NAME");
		break;
	case 3:
		SerialprintP("APN");
		break;
	case 4:
		SerialprintP("APN PW");
		break;
	case 5:
		SerialprintP("APN USER");
		break;
	case 6:
		SerialprintP("PIN");
		break;
	}
}
void BayGPRSInterface::printConfig(void){
	SerialprintlnP("BayGPRS-Config:");
	for(uint8_t i=1;i<=7;i++){
		Serial.print(i);
		SerialprintP(". ");
		printConfigName((i-1));
		SerialprintP(": \"");
		Serial.print(*getConfigPointer((i-1)));
		SerialprintlnP("\"");
	}
	Serial.println();
	SerialprintlnP("press 1-7 to edit, s to save, r to reset");
}

void BayGPRSInterface::serialConfig(void){
	while(true){
		if(_config_mode==0){
			do{
				delay(10);
				Serial.read();
			} while(Serial.available());
			printConfig();
			while(! Serial.available()){}
			_config_mode=Serial.read();
			do{
				delay(10);
				Serial.read();
			} while(Serial.available());
		} else if(_config_mode>='1' && _config_mode<='7'){
			SerialprintP("Please enter new value for ");
			printConfigName((_config_mode-'1'));
			Serial.println();
			while(! Serial.available()){}
			setConfigFromSerial((_config_mode-'1'));
			_config_mode=0;
		} else if(_config_mode=='s'){
			SerialprintP("writing config to EEPROM... ");
			writeConfigToEEPROM();
			SerialprintlnP("done");
			_config_mode=0;
			return;
		} else if(_config_mode=='r'){
			readConfigFromEEPROM();
			_config_mode=0;
		} else {
			SerialprintlnP("unknown key");
			_config_mode=0;

		}
	}
}

char** BayGPRSInterface::getConfigPointer(uint8_t index){
	switch(index){
	case 0:
		return &_url;
		break;
	case 1:
		return &_password;
		break;
	case 2:
		return &_sender;
		break;
	case 3:
		return &_apn;
		break;
	case 4:
		return &_prov_pw;
		break;
	case 5:
		return &_prov_user;
		break;
	case 6:
		return &_pin;
		break;
	}

}

void BayGPRSInterface::setConfigPointers(void){
	uint8_t offset=0;
	_url=_config_buffer;
	char** p;
	for(uint8_t i=1;i<7;i++){
		if(_config_buffer[offset])
			offset+=strlen(_config_buffer+offset)+1;
		else offset++;
		p=getConfigPointer(i);
		*p=_config_buffer+offset;
	}
}

void BayGPRSInterface::setConfig(const char *str,uint8_t index){
	memcpy(_base64buffer,_config_buffer,BayGPRS_CONFIG_SIZE );
	uint8_t offset=0;
	uint8_t offset_old=0;

	for(uint8_t i=0;i<7;i++){
		if(i==index)
			offset+=addToConfigBuffer(offset,str);
		else
			offset+=addToConfigBuffer(offset,_base64buffer+offset_old);
		offset_old+=strlen(_base64buffer+offset_old)+1;
	}
	setConfigPointers();
}

void BayGPRSInterface::setProvider(const char *apn,const char *user,const char *pw){
	setConfig(apn,BayGPRS_CONFIG_APN);
	setConfig(pw,BayGPRS_CONFIG_PROVPW);
	setConfig(user,BayGPRS_CONFIG_PROVUSER);
}

uint8_t BayGPRSInterface::wait_for_available(uint16_t* timeout,int bytes){
	while(i_available()<bytes){
		delay(1);
		(*timeout)--;
		if(*timeout == 0){
		      return 2;
		}
	  }
	return 0;

}



uint8_t BayGPRSInterface::wait_forOK(uint16_t timeout){
	return wait_forPGM(PSTR("OK"),timeout);
}

uint8_t BayGPRSInterface::wait_forPGM(const char* str,uint16_t timeout,uint8_t bytes, char* buffer){
	uint8_t length=0;
	while (true) {
		_pgm_buffer[length]=pgm_read_byte(str);
		if (! _pgm_buffer[length]) break;
	    str++;
	    length++;
	}

	if(wait_for_available(&timeout,length))
	  return 2;
  uint8_t offset=0;
  char c;
  while(true){
	  if(wait_for_available(&timeout))
		  return 2;
	  c=read();
#if BayGPRS_DEBUG_OUTPUT
	Serial.print(c);
#endif

	  if(offset<length){
		  if(c==_pgm_buffer[offset])
		    offset++;
		  else if(offset>0){
			  skipChars();
			return 1;
		  }
	  }

	  if(offset==length){
		  if(bytes && buffer!=NULL){
			  if(wait_for_available(&timeout,bytes))
				  return 2;
			  offset=0;
			  while(offset<bytes){
				  buffer[offset]=read();
				  offset++;
			  }
			  buffer[offset]=0;
		  }
		  skipChars();
		  return 0;
	  }

  }
}


