/*
Router - XBee-GPRS with SD-Card Buffer

GPRS-Shield uses Serial
XBee must get attached to Serial1


ATTENTION!
As the rx-bee may receive data while router is sending, 
it is possible that you loose data.
We recommend to increase SERIAL_BUFFER_SIZE (eg 256)
in arduino-1.0/hardware/arduino/cores/arduino/HardwareSerial.cpp

*/
#include <XBee.h>
#include <BayXBee.h>
#include <BayEOS.h>
#include <SPI.h>
#include <Base64.h>
#include <BayGPRS.h>
#include <BayEOSBuffer.h>
#include <SD.h>
#include <BayEOSBufferSD.h>

#define RX_SERIAL Serial1

/* Please define your BayEOS-Gateway url, password, a name for the GPRS-router and 
the apn of your provider */
BayGPRS client=BayGPRS("http://192.168.0.1:8090/","bayeos","BayGPRS-Router","web.vodafone.de");

XBee xbee_rx = XBee();
BayEOSBufferSD myBuffer;
Rx16Response rx16 = Rx16Response();


uint16_t rx_panid;
	

unsigned long last_alive;

void setup(void){
  pinMode(53, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(4, OUTPUT);
  
 
 
  xbee_rx.setSerial(RX_SERIAL);
  xbee_rx.begin(38400);

  while(! rx_panid){
    rx_panid=getPANID(xbee_rx);
  }
  
  
  client.begin(19200);
  client.connect();
//Use 4 for Ethernet-Shield
// "A11" for Mega-XBee-Shield
  if (!SD.begin(A11)) {
  	client.sendError("No SD.");
  }
  myBuffer = BayEOSBufferSD(10000000);
  client.setBuffer(myBuffer,0);
   
}

void loop(void){
    if(millis()-last_alive>30000){
      last_alive=millis();
      client.startDataFrame(BayEOS_Float32le);
      client.addToPayload((uint8_t) 0);
      client.addToPayload((float) millis()/1000);
      client.sendPayload();
    }
    xbee_rx.readPacket();
   
    if (xbee_rx.getResponse().isAvailable()) {
      // got something
      if (xbee_rx.getResponse().getApiId() == RX_16_RESPONSE ) {
        // got a rx16 packet
        xbee_rx.getResponse().getRx16Response(rx16);
        if (xbee_rx.getResponse().isError()) {
		// get the error code
			client.startFrame(BayEOS_ErrorMessage);
			client.addToPayload("RX-ERROR:"+xbee_rx.getResponse().getErrorCode());
	    } else {
 //         client.startRoutedFrame(rx16.getRemoteAddress16(),rx_panid,rx16.getRssi());
          client.startRoutedFrame(rx16.getRemoteAddress16(),rx_panid);
     	  for(uint8_t i=0; i<rx16.getDataLength();i++){
			client.addToPayload(rx16.getData(i));
		  }
		}
		client.sendOrBuffer();
      }
    }
 
 	client.sendFromBuffer();
}
