/*
 Router - XBee-GPRS with SD-Card Buffer and Watchdog 

 GPRS-Shield uses Serial1
 XBee must get attached to Serial2
 Serial is for debugging and configure

 

 ATTENTION!
 This file will only complile replacing the original 
 arduino-1.0/hardware/arduino/cores/arduino/HardwareSerial.cpp
 arduino-1.0/hardware/arduino/cores/arduino/HardwareSerial.h 
 by the version distributed with the BayEOS arduino libraries

 
 */
#include <EEPROM.h>
#include <XBee.h>
#include <BayXBee.h>
#include <BayEOS.h>
#include <SPI.h>
#include <Base64.h>
#include <BayGPRS.h>
#include <BayDebug.h>
#include <BayEOSBuffer.h>
#include <SdFat.h>
#include <BayEOSBufferSDFat.h>
#include <Sleep.h>
#include <Arduino.h>

#define SerialprintP(x) serialprintPGM(PSTR(x))
#define SerialprintlnP(x) serialprintlnPGM(PSTR(x))


#define SENDING_INTERVAL 120000
#define RX_SERIAL Serial2
#define TX_SERIAL Serial1
#define EEPROM_CHECKPOINT_OFFSET 1024
#define EEPROM_CHECKPOINT_NUMBER 200
#define EEPROM_RESET_POS (EEPROM_CHECKPOINT_OFFSET + 2 + 4 * EEPROM_CHECKPOINT_NUMBER)

/* Please define your BayEOS-Gateway url, password, a name for the GPRS-router and 
 the apn of your provider */

BayGPRS client = BayGPRS("url", "pw", "name",
		"apn", TX_SERIAL);

//BayDebug client;
XBee xbee_rx = XBee();
BayEOSBufferSDFat myBuffer;
RTC_SIM900 myRTC;
Rx16Response rx16 = Rx16Response();

uint16_t rx_panid;

/*
 * Create a huge ring buffer to store incoming RX Packages while
 * arduino is busy with GPRS...
 */
#define RX_BUFFER_SIZE 1024
unsigned char buffer[RX_BUFFER_SIZE];

char SerialInputBuffer[49];

unsigned long last_alive, last_checkpoint, last_send, pos;
uint16_t rx_ok,rx_error,tx_error;
uint8_t rep_tx_error, tx_res;
uint8_t last_rx_rssi;
uint8_t startupframe, startupsend;

//****************************************************************
// Watchdog Interrupt Service / is executed when  watchdog timed out
volatile uint8_t wdcount = 0;
ISR(WDT_vect) {
	wdcount++;
	if(wdcount>30){
          asm volatile (" jmp 0"); //restart programm
        }
}

void setup(void) {
  Serial.begin(9600);
  SerialprintP("FW ");
  SerialprintlnP(__DATE__);
 
  pinMode(53, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(4, OUTPUT);


  SerialprintP("Starting XBee... ");
  //Replace the RX ring buffer with the larger one...
  RX_SERIAL.setRxBuffer(buffer, RX_BUFFER_SIZE);
  xbee_rx.setSerial(RX_SERIAL);
  xbee_rx.begin(38400);
  while (!rx_panid) {
	rx_panid = getPANID(xbee_rx);
  }
  SerialprintlnP("OK");
  SerialprintP("PANID: ");
  Serial.println(rx_panid);
  Serial.println();
   
  startupframe=1;
  startupsend=1;
  client.softSwitch();
  client.setSerialInputBuffer(SerialInputBuffer,49);
  Sleep.setupWatchdog(9); //init watchdog timer to 8 sec
  SerialprintlnP("press 'c' to enter config mode");
  client.readConfigFromEEPROM();
  if (waitForChar('c'))
	client.serialConfig();
  wdcount=0;
  
  Serial.println();
  SerialprintP("Starting GPRS...");
  switch(client.begin(38400)){
    case 0:
      SerialprintlnP("OK");
      break;
    case 1:
      SerialprintlnP("Communication Error");
      break;
    case 2:
      SerialprintlnP("PIN Error");
      break;
  }
  
  Serial.println();
  client.serialPrintlnTime();
  SerialprintlnP("press 'c' to edit time");
  if (waitForChar('c'))
	client.serialSetClock();
  wdcount=0;

  myRTC.adjust(client.now());

  //Use 4 for Ethernet-Shield
  // "A11" for Mega-XBee-Shield
  
  Serial.println();
  if (!SD.begin(4)) {
    SerialprintlnP("No SD.");
    return;
  }

  myBuffer = BayEOSBufferSDFat(2000000000, 1); //Append mode!
  SerialprintP("Buffer Ok - Size:");
  Serial.println(myBuffer.readPos());
  SerialprintlnP("press 'r' to delete Buffer");
  if(myBuffer.readPos()==0)
     EEPROM.write(EEPROM_CHECKPOINT_OFFSET, 0);
  if (waitForChar('r')) {
	SerialprintlnP("deleting Buffer");
	EEPROM.write(EEPROM_CHECKPOINT_OFFSET, 0);
	myBuffer.reset();
	delay(1000);
  }
  wdcount=0;
  
  /* 
  uint8_t* p=(uint8_t*)&pos;
  for (uint8_t i = 0; i < 4; i++) {
     *(p+i)=EEPROM.read(EEPROM_RESET_POS + i);
   }
   if(pos>0 && pos<0xffffffffff){
     SerialprintP("Found reset pos: ");
     SerialprintlnP(pos);
     myBuffer.seekReadPointer(pos);
     pos=0;
    for (uint8_t i = 0; i < 4; i++) {
       EEPROM.write(EEPROM_RESET_POS + i,*(p+i));
    }
   }
  */
  Serial.println();
  SerialprintP("Found ");
  Serial.print(EEPROM.read(EEPROM_CHECKPOINT_OFFSET));
  SerialprintlnP(" checkpoints");
  SerialprintlnP("press c to choose checkpoint");
  if (waitForChar('c')) {
     do{
        delay(5);
        Serial.read();
     } while(Serial.available());
       SerialprintlnP("Enter number of checkpoint to go back - a for all - 0 to exit");
        do{
          delay(10);
       } while(! Serial.available());
     delay(30);
     char buffer[3];
     buffer[0]=Serial.read();
     buffer[1]=Serial.read();
     buffer[2]=0;
    uint8_t nr=0;
    if(buffer[0]=='a'){
       SerialprintlnP("Setting read pointer to 0");
        myBuffer.seekReadPointer(0);
    } else 
      nr=atoi(buffer);
    if(nr){
      SerialprintP("Setting read pointer to checkpoint ");
      Serial.println(nr);
      setToCheckpoint(nr);
    }
  }
  wdcount=0;

  myBuffer.setRTC(myRTC, 0); //Relative Mode...
  client.setBuffer(myBuffer, 0);


  Serial.println();
  SerialprintP("Connecting... ");
  while(client.connect()){
    SerialprintlnP("failed");
    SerialprintP("Connecting... ");
  }  
  SerialprintlnP("OK");
  Serial.println();
  SerialprintlnP("Setup ok :-)");
  Serial.println();
  
  
  client.startFrame(BayEOS_Message);
  client.addToPayload("Router started - FW");
  client.addToPayload(__DATE__);
  client.writeToBuffer();
}

void loop(void) {
	wdcount = 0; //clear watchdog count!
	if ((millis() - last_checkpoint) > ((unsigned long)1000 * 3600 * 6)) { //Save read_pos to EEPROM every Day
		last_checkpoint = millis();
		setCheckpoint(myBuffer.readPos());
	}

	if ((millis() - last_alive) > SENDING_INTERVAL || startupframe) {
                startupframe=0;
		last_alive = millis();
		client.startDataFrame(BayEOS_Float32le);
		client.addChannelValue(millis() / 1000);
		client.addChannelValue(myBuffer.writePos());
		client.addChannelValue(myBuffer.readPos());
		client.addChannelValue(tx_error);
		client.addChannelValue(rx_ok);
		client.addChannelValue(rx_error);
                client.addChannelValue(EEPROM.read(EEPROM_CHECKPOINT_OFFSET));
                client.addChannelValue(myRTC.now().get());
                client.addChannelValue(client.now().get());
                client.addChannelValue(client.getRSSI());
                client.addChannelValue((float)analogRead(A15)/1024*5*10);
  	        client.writeToBuffer();
        }
        
	if ((millis() - last_send) > SENDING_INTERVAL || myBuffer.available()>5000 || startupsend) {
		last_send = millis();
        
		SerialprintP("Sending via GPRS... ");
		if ( (tx_res=client.sendMultiFromBuffer()) ){
			SerialprintlnP("failed");
                        tx_error++;
                        rep_tx_error++;
                        client.startFrame(BayEOS_ErrorMessage);
                        client.addToPayload("TX-ERROR:");
                        client.addToPayload((uint8_t)(tx_res+'0'));
                        client.writeToBuffer();
                        
                } else {
                        rep_tx_error=0;
                        startupsend=0;
			SerialprintlnP("OK");
                        unsigned long simTime=client.now().get();
                        unsigned long rtcTime=myRTC.now().get();
                        if((simTime>rtcTime && (simTime-rtcTime)<120)
                         || (simTime>rtcTime && (rtcTime-simTime)<120) ){
                           SerialprintlnP("setting RTC");
		           myRTC.adjust(DateTime(simTime));
                         } 

                }
                if(rep_tx_error%5==4){
                  client.softSwitch();
                  client.startFrame(BayEOS_Message);
                  client.addToPayload("TX-ERROR SoftSwitch");
                  client.writeToBuffer();

                }
/* 
if(rep_tx_error>30){
                    SerialprintlnP("resetting!");
                  client.startFrame(BayEOS_Message);
                  client.addToPayload("TX-ERROR Reset");
                  client.writeToBuffer();
                    pos=myBuffer.readPos();
                    uint8_t* p=(uint8_t*)&pos;
                    for (uint8_t i = 0; i < 4; i++) {
	               EEPROM.write(EEPROM_RESET_POS + i,*(p + i));
                    }
                    asm volatile (" jmp 0"); //restart programm 
                 }
*/
	}
        
   handle_RX_data();
}

void handle_RX_data(void){
    while(RX_SERIAL.available()){        
	xbee_rx.readPacket();

	if (xbee_rx.getResponse().isAvailable()) {
		// got something
		if (xbee_rx.getResponse().getApiId() == RX_16_RESPONSE) {
			// got a rx16 packet
			xbee_rx.getResponse().getRx16Response(rx16);
			if (xbee_rx.getResponse().isError()) {
                                rx_error++;
				// get the error code
				client.startFrame(BayEOS_ErrorMessage);
				client.addToPayload("RX-ERROR:");
				client.addToPayload((uint8_t)(xbee_rx.getResponse().getErrorCode() + '0'));
			} else {
                                rx_ok++;
				SerialprintlnP("Got RX-Packet");
                                last_rx_rssi=rx16.getRssi();
				client.startRoutedFrame(rx16.getRemoteAddress16(), rx_panid,
						last_rx_rssi);
				for (uint8_t i = 0; i < rx16.getDataLength(); i++) {
					client.addToPayload(rx16.getData(i));
				}
			}
			client.writeToBuffer();
		}
	}
    }
  
}

uint8_t waitForChar(char c) {
	int timeout = 6000;
	do {
		if (Serial.available() && Serial.read() == c) {
			Serial.println();
			return 1;
		}
		if (timeout % 1000 == 0) {
			Serial.print((timeout / 1000));
			SerialprintP("..");
		}
		timeout--;
		delay(1);
	} while (timeout > 0);
	Serial.println();
	return 0;

}

void setToCheckpoint(uint8_t nr){
   uint8_t next = EEPROM.read(EEPROM_CHECKPOINT_OFFSET + 1);
   if(next>=nr) next-=nr;
   else next+=EEPROM_CHECKPOINT_NUMBER-nr;
   uint8_t* p=(uint8_t*)&pos;
   for (uint8_t i = 0; i < 4; i++) {
  	*(p+i)=EEPROM.read(EEPROM_CHECKPOINT_OFFSET + 2 + (next * 4) + i);
   }
   myBuffer.seekReadPointer(pos);
   
   
}

void setCheckpoint(unsigned long pos) {
  uint8_t num = EEPROM.read(EEPROM_CHECKPOINT_OFFSET);
  if (num == 0xff)
	num = 0;
  uint8_t next = EEPROM.read(EEPROM_CHECKPOINT_OFFSET + 1);
  if (next == 0xff)
    next = 0;
  if(num==0)
    next=0;
  uint8_t* p=(uint8_t*)&pos;
  for (uint8_t i = 0; i < 4; i++) {
	EEPROM.write(EEPROM_CHECKPOINT_OFFSET + 2 + (next * 4) + i,
				*(p + i));
  }
  next++;
  if (next >= EEPROM_CHECKPOINT_NUMBER)
    next = 0;
  if (num < EEPROM_CHECKPOINT_NUMBER)
    num++;
    SerialprintP("Setting Checkpoint: ");
    
    Serial.print(num);
    SerialprintP("/");
    Serial.println(next);
  EEPROM.write(EEPROM_CHECKPOINT_OFFSET, num);
  EEPROM.write(EEPROM_CHECKPOINT_OFFSET + 1, next);

}


void serialprintPGM(const char *str){
	char c;
	while (true) {
		c=pgm_read_byte(str);
		if (!c) break;
	    Serial.write(c);
	    str++;
	}
}

void serialprintlnPGM(const char *str){
	serialprintPGM(str);
	Serial.println();
}



