/*
 Router - XBee-GPRS with SD-Card Buffer and Watchdog 

 for ITEAD Wireless board
 

 ATTENTION!
 This file will only complile replacing the original 
 arduino-1.0/hardware/arduino/cores/arduino/HardwareSerial.cpp
 arduino-1.0/hardware/arduino/cores/arduino/HardwareSerial.h 
 by the version distributed with the BayEOS arduino libraries
 
 
 */
#include <EEPROM.h>
#include <XBee.h>
#include <BayXBee.h>
#include <BayEOS.h>
#include <SPI.h>
#include <Base64.h>
#include <BayGPRS.h>
#include <BayGPRSsoftserial.h>
#include <BayDebug.h>
#include <BayEOSBuffer.h>
#include <SdFat.h>
#include <BayEOSBufferSDFat.h>
#include <SoftwareSerial.h>
#include <Sleep.h>
#include <Arduino.h>

#define SERIAL_DEBUG 0
#define RX_SERIAL Serial
#define RX_SLEEP_PIN A5
#define BAT_POWER_PIN A0
#define SENDING_INTERVAL 300000

/* Please define your BayEOS-Gateway url, password, a name for the GPRS-router and 
 the apn of your provider */

BayGPRSsoftserial client = BayGPRSsoftserial(2,3,6);

//BayDebug client;
XBee xbee_rx = XBee();
BayEOSBufferSDFat myBuffer;
Rx16Response rx16 = Rx16Response();

uint16_t rx_panid;

#define SerialprintP(x) serialprintPGM(PSTR(x))
#define SerialprintlnP(x) serialprintlnPGM(PSTR(x))
unsigned char* RXbuffer;


unsigned long last_alive, last_send;
uint16_t rx_ok,rx_error,tx_error;
uint8_t rep_tx_error, tx_res;
uint8_t last_rx_rssi;
uint8_t startupframe, startupsend;

//****************************************************************
// Watchdog Interrupt Service / is executed when  watchdog timed out
volatile uint8_t wdcount = 0;
ISR(WDT_vect) {
	wdcount++;
	if(wdcount>30){
          asm volatile (" jmp 0"); //restart programm
        }
}

void setup(void) {
  pinMode(RX_SLEEP_PIN,OUTPUT);
  digitalWrite(RX_SLEEP_PIN,LOW);
  xbee_rx.setSerial(RX_SERIAL);
  xbee_rx.begin(38400);
#if SERIAL_DEBUG
  SerialprintlnP("Starting");
#endif

  pinMode(10, OUTPUT);
  pinMode(4, OUTPUT);
  if (!SD.begin(10)) {
    SerialprintlnP("No SD.");
    delay(1000);
    return;
  }
#if SERIAL_DEBUG
  SerialprintlnP("SD OK");
  delay(10);
#endif

  client.readConfigFromFile("GPRS");
//  client.readConfigFromEEPROM();
//  client.printConfig();
#if SERIAL_DEBUG
  SerialprintlnP("Config OK");
  rx_panid=10002;
#endif

  while (!rx_panid) {
	rx_panid = getPANID(xbee_rx);
  }
  // Set XBEE to sleep mode 
  digitalWrite(RX_SLEEP_PIN,HIGH);
   
  startupframe=1;
  startupsend=1;
  client.softSwitch();
  Sleep.setupWatchdog(9); //init watchdog timer to 8 sec

  
  //19200 is the maximum speed for SoftSerial!
  client.begin(19200);
    
 
  myBuffer = BayEOSBufferSDFat(2000000000, 1); //Append mode!
#if SERIAL_DEBUG
  SerialprintlnP("d");
#endif
  
  client.setBuffer(myBuffer, 0);

/*
 * Create a huge ring buffer to store incoming RX Packages while
 * arduino is busy with GPRS...
 */
  int max_length=512;
  while((RXbuffer=(uint8_t *)malloc(max_length))==NULL){
		max_length-=10;
  }

#if SERIAL_DEBUG
  SerialprintP("RXBuffer:");
  Serial.println(max_length);
#endif
  
  //Replace the RX ring buffer with the larger one...
  RX_SERIAL.setRxBuffer(RXbuffer, max_length);



#if SERIAL_DEBUG
  SerialprintlnP("Connecting");
#endif
  while(client.connect()){
#if SERIAL_DEBUG
    SerialprintlnP("failed");
    SerialprintP("Connecting... ");
#endif
  }  
#if SERIAL_DEBUG
  SerialprintlnP("OK");
  Serial.println();
  SerialprintlnP("Setup ok :-)");
  Serial.println();
#endif
  //Wake up XBEE again...
  digitalWrite(RX_SLEEP_PIN,LOW);
  
}

void loop(void) {
	wdcount = 0; //clear watchdog count!

	if ((millis() - last_alive) > SENDING_INTERVAL || startupframe) {
                startupframe=0;
		last_alive = millis();
		client.startDataFrame(BayEOS_Float32le);
		client.addChannelValue(millis() / 1000);
		client.addChannelValue(myBuffer.writePos());
		client.addChannelValue(myBuffer.readPos());
		client.addChannelValue(tx_error);
		client.addChannelValue(rx_ok);
		client.addChannelValue(rx_error);
                client.addChannelValue(client.now().get());
                client.addChannelValue(client.getRSSI());
                client.addChannelValue((float)analogRead(BAT_POWER_PIN)/1024*3.3*10);
  	        client.writeToBuffer();
        }
        
	if ((millis() - last_send) > SENDING_INTERVAL || myBuffer.available()>5000 || startupsend) {
                //XBEE sleep mode
                digitalWrite(RX_SLEEP_PIN,HIGH);
		last_send = millis();
        
#if SERIAL_DEBUG
		SerialprintP("Sending via GPRS... ");
#endif
		if ( (tx_res=client.sendMultiFromBuffer()) ){
#if SERIAL_DEBUG
			SerialprintlnP("failed");
#endif
                        tx_error++;
                        rep_tx_error++;
                        client.startFrame(BayEOS_ErrorMessage);
                        client.addToPayload("TX:");
                        client.addToPayload((uint8_t)(tx_res+'0'));
                        client.writeToBuffer();
                        
                } else {
                        rep_tx_error=0;
                        startupsend=0;
#if SERIAL_DEBUG
			SerialprintlnP("OK");
#endif
                }
                if(rep_tx_error%5==4){
                  client.softSwitch();

                }
                //XBEE sleep mode off
                digitalWrite(RX_SLEEP_PIN,LOW);

	}
        
   handle_RX_data();
}

void handle_RX_data(void){
    while(RX_SERIAL.available()){        
	xbee_rx.readPacket();

	if (xbee_rx.getResponse().isAvailable()) {
		// got something
		if (xbee_rx.getResponse().getApiId() == RX_16_RESPONSE) {
			// got a rx16 packet
			xbee_rx.getResponse().getRx16Response(rx16);
			if (xbee_rx.getResponse().isError()) {
                                rx_error++;
				// get the error code
			} else {
                                rx_ok++;
#if SERIAL_DEBUG
				SerialprintlnP("Got RX-Packet");
#endif
                                last_rx_rssi=rx16.getRssi();
				client.startRoutedFrame(rx16.getRemoteAddress16(), rx_panid,
						last_rx_rssi);
				for (uint8_t i = 0; i < rx16.getDataLength(); i++) {
					client.addToPayload(rx16.getData(i));
				}
			}
			client.writeToBuffer();
		}
	}
    }
  
}


void serialprintPGM(const char *str){
	char c;
	while (true) {
		c=pgm_read_byte(str);
		if (!c) break;
	    Serial.write(c);
	    str++;
	}
}

void serialprintlnPGM(const char *str){
	serialprintPGM(str);
	Serial.println();
}



